#include "biasedfish.h"
#include "environ.h"


const int FORWARD_BIAS = 2;


// constructor

BiasedFish::BiasedFish(int id, const Position & pos)
  : MemoryFish(id, pos)                    // invoke the parent constructor
{}

// public accessing functions

Position BiasedFish::NextLocation(const Environment & env) {
  int dr = Location().Row() - OldLocation().Row();
  int dc = Location().Col() - OldLocation().Col();

  //  cout << "Fish " << ShowMe() << " had dr=" << dr << " and dc=" << dc << endl;

  Position forward(Location().Row() + dr, Location().Col() + dc);
  Position reverse(Location().Row() - dr, Location().Col() - dc);
  Position left(Location().Row() + dc, Location().Col() - dr);
  Position right(Location().Row() - dc, Location().Col() + dr);

  // we will draw straws to see which direction wins
  int straws = 0;

  if (env.IsEmpty(forward))
    straws += FORWARD_BIAS;             // more likely to go forward then turn
  if (env.IsEmpty(left))
    straws += 1;
  if (env.IsEmpty(right))
    straws += 1;

  if (straws == 0) {           
    // cannot move forward, left, or right; perhaps backward?
    if (env.IsEmpty(reverse))
      return reverse;
    else
      return Location();     // no move was possible; return current location
  } else {
    int choice = randomVals.RandInt(0, straws-1);

    if (env.IsEmpty(forward)) {
      choice -= FORWARD_BIAS;             // twice as likely as a turn
      if (choice < 0)
        return forward;
    }

    if (env.IsEmpty(left)) {
      choice -= 1;
      if (choice < 0)
	return left;
    }

    return right;
  }
}
