// AP Computer Science Marine Biology Case Study program
// Copyright (C) 2000  College Board and Educational Testing Service
// Copyright (C) 2013  Michael H. Goldwasser

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// fish.cpp - copyright statement added 5/31/2000
// changed to use std string class, mhg 1/8/2013

#include "environ.h"
#include "fish.h"
#include "nbrhood.h"
#include "position.h"
#include "utils.h"

// constructors

Fish::Fish(int id, const Position & pos)
  : myId(id),
    myPos(pos),
    randomVals()
// postcondition: Location() returns pos, Id() returns id,
{

}

// public accessing functions

int Fish::Id() const
// postcondition: returns id number of fish
{
  return myId;
}

Position Fish::Location() const
// postcondition: returns current fish position
{
  return myPos;
}

string Fish::ToString() const
// postcondition: returns a stringized form of Fish
{
  return IntToString(myId) + " " + myPos.ToString();
}

// public modifying functions

void Fish::Move(Environment & env)
// precondition:  Fish stored in env at Location()
// postcondition: Fish has moved to a new location in env (if possible)
{
  Position newPos = NextLocation(env);
  if (newPos != myPos) {
    Position oldPos = myPos;
    DebugPrint(1, "Fish at " + myPos.ToString() + " moves to " + newPos.ToString());
    myPos = newPos;
    env.Update(oldPos, *this); // *this means this fish
  } else {
    DebugPrint(1, "Fish " + ToString() + " didn't move.");
  }
}


Position Fish::NextLocation(const Environment & env) {
  Neighborhood nbrs = EmptyNeighbors(env, myPos);
  DebugPrint(3, nbrs.ToString());

  if (nbrs.Size() > 0)
  {
    // there were some empty neighbors, so randomly choose one
    return nbrs.Select(randomVals.RandInt(0, nbrs.Size() - 1));
  } else {
    return myPos;
  }
}


char Fish::ShowMe() const
// postcondition: returns a character that can make me visible
{
  if (1 <= Id() && Id() <= 26)
  {
    return 'A' + (Id() - 1);
  }
  return '*';
}

// private helper functions

Neighborhood Fish::EmptyNeighbors(const Environment & env,
                                  const Position & pos) const
//  precondition:  pos is in the grid being modelled
//  postcondition: returns a Neighborhood of pos of empty positions
{
  Neighborhood nbrs;

  AddIfEmpty(env, nbrs, pos.North());
  AddIfEmpty(env, nbrs, pos.South());
  AddIfEmpty(env, nbrs, pos.East());
  AddIfEmpty(env, nbrs, pos.West());
  
  return nbrs;
}

void Fish::AddIfEmpty(const Environment & env,
                      Neighborhood & nbrs, const Position & pos) const
// postcondition: pos is added to nbrs if pos in env and empty
{
  if (env.IsEmpty(pos))
  {
    nbrs.Add(pos);
  }
}

// free functions

ostream & operator << (ostream & out, const Fish & fish)
// postcondition: fish inserted onto stream out
{
  out << fish.ToString();
  return out;
}




